﻿using Netina.Domain.Entities.Blogs;

namespace Netina.AdminPanel.PWA.Pages;

public class BlogsPageViewModel : BaseViewModel<ObservableCollection<BlogSDto>>
{
    private readonly NavigationManager _navigationManager;
    private readonly ISnackbar _snackbar;
    private readonly IUserUtility _userUtility;
    private readonly IDialogService _dialogService;
    private readonly IRestWrapper _restWrapper;

    public string Search = string.Empty;
    public int CurrentPage = 0;
    public int PageCount = 1;
    public int TotalItems = 0;
    public BlogsPageViewModel(NavigationManager navigationManager, ISnackbar snackbar, IUserUtility userUtility, IRestWrapper restWrapper, IDialogService dialogService) : base(userUtility)
    {
        _navigationManager = navigationManager;
        _snackbar = snackbar;
        _userUtility = userUtility;
        _restWrapper = restWrapper;
        _dialogService = dialogService;
    }

    public override async Task InitializeAsync()
    {
        try
        {
            IsProcessing = true;
            PageDto.Clear();
            var dto = await _restWrapper.BlogRestApi.ReadAll(CurrentPage);
            dto.Blogs.ForEach(d => PageDto.Add(d));
            if (PageDto.Count == 20)
                PageCount = 2;
            TotalItems = dto.Pager.TotalItems;
        }
        catch (ApiException ex)
        {
            var exe = await ex.GetContentAsAsync<ApiResult>();
            _snackbar.Add(exe != null ? exe.Message : ex.Content, Severity.Error);
        }
        catch (Exception e)
        {
            _snackbar.Add(e.Message, Severity.Error);
        }
        finally
        {

            IsProcessing = false;
        }
        await base.InitializeAsync();
    }

    public async Task ChangePageAsync(int page)
    {
        CurrentPage = page - 1;
        if (CurrentPage > PageCount - 2)
        {

            try
            {
                IsProcessing = true;

                GetBlogsResponseDto dto = new GetBlogsResponseDto();
                if (Search.IsNullOrEmpty())
                {
                    dto = await _restWrapper.BlogRestApi.ReadAll(CurrentPage);
                }
                else
                {
                    dto = await _restWrapper.BlogRestApi.ReadAll(CurrentPage, Search);
                }

                dto.Blogs.ForEach(d => PageDto.Add(d));
                if (PageDto.Count % 20 == 0)
                    PageCount = CurrentPage + 2;

            }
            catch (ApiException ex)
            {
                var exe = await ex.GetContentAsAsync<ApiResult>();
                _snackbar.Add(exe != null ? exe.Message : ex.Content, Severity.Error);
            }
            catch (Exception e)
            {
                _snackbar.Add(e.Message, Severity.Error);
            }
            finally
            {

                IsProcessing = false;
            }
        }
    }

    public async Task AddBlogClicked()
    {
        DialogOptions maxWidth = new DialogOptions() { MaxWidth = MaxWidth.Medium, FullWidth = true, DisableBackdropClick = true };
        await _dialogService.ShowAsync<BlogActionDialogBox>("افزودن بلاگ جدید", maxWidth);
    }

    public async Task EditBlogClicked(BlogSDto blog)
    {
        DialogOptions maxWidth = new DialogOptions() { MaxWidth = MaxWidth.Medium, FullWidth = true, DisableBackdropClick = true };
        var parameters = new DialogParameters<BlogActionDialogBox>();
        parameters.Add(x => x.Blog, blog);
        await _dialogService.ShowAsync<BlogActionDialogBox>($"ویرایش بلاگ {blog.Title}", parameters, maxWidth);
    }

    public async Task DeleteBlogAsync(Guid selectedCategoryId)
    {
        var reference = await _dialogService.ShowQuestionDialog($"آیا از حذف بلاگ اطمینان دارید ?");
        var result = await reference.Result;
        if (!result.Canceled)
        {

            try
            {

                IsProcessing = true;
                var token = await _userUtility.GetBearerTokenAsync();
                await _restWrapper.CrudDtoApiRest<Blog, BlogSDto, Guid>(Address.BlogController)
                    .Delete(selectedCategoryId, token);
                _snackbar.Add("حذف بلاگ با موفقیت انجام شد", Severity.Success);

            }
            catch (ApiException ex)
            {
                var exe = await ex.GetContentAsAsync<ApiResult>();
                _snackbar.Add(exe != null ? exe.Message : ex.Content, Severity.Error);
            }
            catch (Exception e)
            {
                _snackbar.Add(e.Message, Severity.Error);
            }
            finally
            {

                IsProcessing = false;
            }
        }
    }

    public async Task SearchChanged(string search)
    {
        if (search.IsNullOrEmpty() && !Search.IsNullOrEmpty())
            await InitializeAsync();
        Search = search;
    }
    public async Task SearchAsync()
    {
        try
        {
            if (Search.IsNullOrEmpty())
                throw new AppException("نام بلاگ برای جست جو وارد نشده است");
            IsProcessing = true;
            CurrentPage = 0;
            PageCount = 1;
            PageDto.Clear();
            var dto = await _restWrapper.BlogRestApi.ReadAll(CurrentPage, Search);
            dto.Blogs.ForEach(d => PageDto.Add(d));
            if (PageDto.Count == 20)
                PageCount = 2;
        }
        catch (ApiException ex)
        {
            var exe = await ex.GetContentAsAsync<ApiResult>();
            _snackbar.Add(exe != null ? exe.Message : ex.Content, Severity.Error);
        }
        catch (Exception e)
        {
            _snackbar.Add(e.Message, Severity.Error);
        }
        finally
        {

            IsProcessing = false;
        }
    }
}